package eaadapter.ea2ecore.postprocessings;

import java.io.File;
import java.math.BigInteger;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Map.Entry;

import javax.activation.MimeType;
import javax.xml.datatype.XMLGregorianCalendar;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.impl.EClassImpl;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import eaadapter.common.util.ModelConstants;
import eaadapter.ea2ecore.PostProcessingTemplate;
import eaadapter.ea2ecore.postprocessings.util.EcoreUtil;

public class ReplaceReferences extends PostProcessingTemplate {
	
	private static final String EASTADL_STUB_ECORE = "eastadlStub.ecore";
	private static final String AUTOSAR_STUB_ECORE = "autosarStub.ecore";
	
	//a map of EAST-ADL class name and its package name
	private static Map<String, String> eastadlMapping;	
	static {
		eastadlMapping = new HashMap<String, String>();
		eastadlMapping.put("Allocation", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("AnalysisFunctionPrototype", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("AnalysisLevel", "systemmodeling"); //"eastadl21.east_adl.structure.systemmodeling");
		eastadlMapping.put("DesignFunctionPrototype", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("DesignLevel", "systemmodeling"); //"eastadl21.east_adl.structure.systemmodeling");
		eastadlMapping.put("FeatureModel", "featuremodeling"); //"eastadl21.east_adl.structure.featuremodeling");
		eastadlMapping.put("FunctionConnector", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("FunctionPort", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("FunctionPrototype", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("FunctionType", "functionmodeling");//"eastadl21.east_adl.structure.functionmodeling");
		eastadlMapping.put("HardwareComponentPrototype", "hardwaremodeling"); //"eastadl21.east_adl.structure.hardwaremodeling");
		eastadlMapping.put("HardwareComponentType", "hardwaremodeling"); //"eastadl21.east_adl.structure.hardwaremodeling" );
		eastadlMapping.put("HardwarePin", "hardwaremodeling"); //"eastadl21.east_adl.structure.hardwaremodeling" );
		eastadlMapping.put("ImplementationLevel", "systemmodeling"); //"eastadl21.east_adl.structure.systemmodeling" );
		eastadlMapping.put("Requirement", "requirements"); //"eastadl21.east_adl.requirements" );
		eastadlMapping.put("SystemModel", "systemmodeling"); //"eastadl21.east_adl.structure.systemmodeling" );
		eastadlMapping.put("VehicleFeature", "vehiclefeaturemodeling"); //"eastadl21.east_adl.structure.vehiclefeaturemodeling" );
		eastadlMapping.put("AnalysisFunctionType", "functionmodeling"); //"eastadl21.east_adl.structure.functionmodeling" );
		eastadlMapping.put("Environment", "environment"); //"eastadl21.east_adl.environment" );
		eastadlMapping.put("DesignFunctionType", "functionmodeling"); //"eastadl21.east_adl.structure.functionmodeling" );
	}	
	
	//a map of AUTOSAR class name and its package name
		private static Map<String, String> autosarMapping;	
		static {
			autosarMapping = new HashMap<String, String>();
			autosarMapping.put("BswModuleDescription", "bswoverview");//"autosar40.bswmodule.bswoverview");
			autosarMapping.put("BswModuleEntry", "bswinterfaces");//"autosar40.bswmodule.bswinterfaces");
			autosarMapping.put("ClientServerOperation", "portinterface"); //"autosar40.swcomponent.portinterface");
			autosarMapping.put("SwComponentPrototype", "composition");//"autosar3x.swcomponent.composition");
			autosarMapping.put("CompositionSwComponentType", "composition"); //autosar40.swcomponent.composition
			autosarMapping.put("HwElement", "ecuresource"); //autosar3x.ecuresource
			autosarMapping.put("HwElementPrototype", "ecuresource"); // replace with HwElement TODO replace with real AUTOSAR element
			autosarMapping.put("HwElementType", "ecuresource"); // replace with HwElement TODO replace with real AUTOSAR element
			autosarMapping.put("ModeDeclaration", "modedeclaration"); //autosar40.commonstructure.modedeclaration
			autosarMapping.put("ModeDeclarationGroupPrototype", "modedeclaration"); //autosar40.commonstructure.modedeclaration
			autosarMapping.put("PortPrototype", "components"); //autosar40.swcomponent.components
			autosarMapping.put("RunnableEntity", "swcinternalbehavior"); //autosar40.swcomponent.swcinternalbehavior
			autosarMapping.put("SwComponentType", "components"); //autosar40.swcomponent.components
			autosarMapping.put("System", "system"); //autosar40.system
			autosarMapping.put("VariableDataPrototype", "dataprototypes"); //autosar40.swcomponent.datatype.dataprototypes
			autosarMapping.put("ComponentInCompositionInstanceRef", "instancerefs"); //autosar40.swcomponent.composition.instancerefs
		}	
	
	private File modelFile = null;
	
	private Map<EClass, Collection<String>> fMissingEClasses = new HashMap<EClass, Collection<String>>();

	public ReplaceReferences (File modelFile) {
		this.modelFile = modelFile;
	}

	@Override
	public void execute() {
		
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getURIConverter().getURIMap().putAll(EcorePlugin.computePlatformURIMap());
		
		IPath genModelPath = new Path(modelFile.getAbsolutePath());
		IFile iFile = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(
				genModelPath);
		IContainer folder = iFile.getParent();
		IResource eastadlStubEcoreFile = folder.findMember(EASTADL_STUB_ECORE);
		
		if (eastadlStubEcoreFile == null || !eastadlStubEcoreFile.exists()) {
			return;
		}		
		replaceEastadlReferences(eastadlStubEcoreFile, resourceSet);
		
        IResource autosarStubEcoreFile = folder.findMember(AUTOSAR_STUB_ECORE);		
		if (autosarStubEcoreFile == null || !autosarStubEcoreFile.exists()) {
			return;
		}
		
		replaceAutosarReferences(autosarStubEcoreFile, resourceSet);
//		resourceSet.getResources().add(targetEcore);
		
//		List<EClass> gEClasses = EcoreUtil.getEAllEClasses(eastadlEcoreRes);
//		String modelId = getModelId(targetEcore);
//		for (EClass gEClass : gEClasses) {
//			Iterable<EClass> tgtEClasses = getTargetEClasses(gEClass, modelId, targetEcore);
//			for (EClass tgtEClass : tgtEClasses) {
//				EAnnotation mergeConfig = gEClass.getEAnnotation(MERGE_CONFIG);
//				if (mergeConfig != null) {
//					String mode = mergeConfig.getDetails().get(MODE);
//					if (REPLACE.equals(mode)) {
//						tgtEClass.getESuperTypes().clear();
//					}
//				}
//				tgtEClass.getESuperTypes().add(gEClass);
//			}
//		}
//		handleMissingEClasses();	
		
		
	}
	
	private void replaceEastadlReferences(IResource eastadlEcoreFile, ResourceSet resourceSet){
		URI ecoreURI = URI.createPlatformResourceURI(eastadlEcoreFile.getFullPath().toString(), true);
		Resource eastadlEcoreRes = resourceSet.getResource(ecoreURI, true);
		
		ecoreURI = URI.createFileURI(modelFile.getAbsolutePath().toString());
		Resource targetEcore = Resource.Factory.Registry.INSTANCE.getFactory(ecoreURI).createResource(ecoreURI);
		targetEcore.getContents().add(model);
		
		resourceSet.getResources().add(targetEcore);
		
		EPackage pkg = findEPackageByName("eastadlreferences");
		EList<EClassifier> eastadlReferenceList = pkg.getEClassifiers();

		if (pkg != null) {			
			TreeIterator<EObject> iter = model.eAllContents();
			while (iter.hasNext()) {
				EObject element = (EObject) iter.next();
				if (element instanceof EReference) {
					EReference eRef = (EReference) element;
					EClassifier eType = eRef.getEType();
					if (eastadlReferenceList.contains(eType)) {
						EClassifier eastadlElement = getMappedEastadlElement(eType, eastadlEcoreRes);
						eRef.setEType(eastadlElement);
						eRef.setEOpposite(null);
						logger.info("Replacing EAST-ADL reference " + eType.getName());
					}
				}
			}
					
		}
	}
	
	private EClassifier getMappedEastadlElement(EClassifier eReferenceType, Resource eastadlEcoreRes){
		String referenceTypeName = eReferenceType.getName();
		if (referenceTypeName != null) {
			String packageName = eastadlMapping.get(referenceTypeName);
			if (packageName != null) {
				 EPackage eastadlRootPackage = getRootPackage(eastadlEcoreRes);
				 EPackage ePackage = getEPackageByName(eastadlRootPackage, packageName);
				 EClassifier eClassifier = getEClassifier(ePackage, referenceTypeName);
				 return eClassifier;
			} else {
				logger.error("No mapping for EAST-ADL reference " + referenceTypeName + " available! The instanceClassName for " + referenceTypeName + " could not set!");
			}
		}
		
		return null;
	}
	
	protected EPackage getEPackageByName(EPackage rootPackage, String name) {
		TreeIterator<EObject> iter = rootPackage.eAllContents();
		while (iter.hasNext()) {
			EObject element = (EObject) iter.next();
			if (element instanceof EPackage) {
				if (((EPackage) element).getName().equals(name))
					return (EPackage) element;
			}
		}
		return null;
	}
	
	protected EClassifier getEClassifier(EPackage ePackage, String eClassifiername) {
		TreeIterator<EObject> iter = ePackage.eAllContents();
		while (iter.hasNext()) {
			EObject element = (EObject) iter.next();
			if (element instanceof EClassifier) {
				if (((EClassifier) element).getName().equals(eClassifiername))
					return (EClassifier) element;
			}
		}
		return null;
	}
	
//	private Collection<EClass> getTargetEClasses(EClass gEClass, String modelId, Resource targetModel) {
//		Collection<String> tgtEClassNames = getTargetEClassNames(gEClass, modelId);
//		Collection<EClass> tgtEClasses = new ArrayList<EClass>();
//		if (isNoneMapping(tgtEClassNames)) {
//			return Collections.<EClass> singleton(new NullEClass());
//		}
//		for (String tgtEClassName : tgtEClassNames) {
//			EClass tgtEClass = EcoreUtil.findEClass(tgtEClassName, targetModel);
//			if (tgtEClass != null) {
//				tgtEClasses.add(tgtEClass);
//			} else {
//				if (!fMissingEClasses.containsKey(gEClass)) {
//					fMissingEClasses.put(gEClass, new ArrayList<String>());
//				}
//				fMissingEClasses.get(gEClass).add(tgtEClassName);
//			}
//		}
//		return tgtEClasses;
//	}
//	
//	private Collection<String> getTargetEClassNames(EClass gEClass, String modelId) {
//		Collection<String> mappedNames = getMappedTargetEClassNames(gEClass, modelId);
//		if (!mappedNames.iterator().hasNext()) {
//			mappedNames.add(gEClass.getName().substring(1));
//		}
//		return mappedNames;
//	}
//	
//	private Collection<String> getMappedTargetEClassNames(EClass gEClass, String modelId) {
//		Collection<String> mappedTgtNames = new ArrayList<String>();
//		EAnnotation mapping = gEClass.getEAnnotation(CLASS_MAPPING);
//		if (mapping != null) {
//			String mappedTgtNameString = mapping.getDetails().get(modelId);
//			if (mappedTgtNameString != null) {
//				StringTokenizer tokenizer = new StringTokenizer(mappedTgtNameString, TGT_ECLASS_NAME_SEPARATOR);
//				while (tokenizer.hasMoreTokens()) {
//					mappedTgtNames.add(tokenizer.nextToken().trim());
//				}
//			}
//		}
//		return mappedTgtNames;
//	}
//	
//	private boolean isNoneMapping(Collection<String> tgtEClassNames) {
//		return tgtEClassNames.size() == 1 && NONE.equalsIgnoreCase(tgtEClassNames.iterator().next());
//	}
//	
//	private String getModelId(Resource targetModel) {
//		return getLastSegment(getNsURI(targetModel));
//	}
//
//	private String getNsURI(Resource targetModel) {
//		EObject eObj = targetModel.getContents().get(0);
//		if (!(eObj instanceof EPackage)) {
//			throw new IllegalArgumentException("Could not identify target-model id. No root EPackage found for target-model!");
//		}
//		return ((EPackage) eObj).getNsURI();
//	}
//
//	private String getLastSegment(String nsURI) {
//		int index = nsURI.lastIndexOf(EA_MODEL_LAST_SGMT_PREFIX);
//		if (index == -1) {
//			throw new IllegalArgumentException("Target-model is not an AUTOSAR meta-model!");
//		}
//		return nsURI.substring(index + 1);
//	}
	
	private void handleMissingEClasses() {
		logMissingEClasses();
	}

	private void logMissingEClasses() {
		for (Entry<EClass, Collection<String>> missingEClasses : fMissingEClasses.entrySet()) {
			for (String tgtEClassName : missingEClasses.getValue()) {
				String logMsg = "The target EClass \"" + tgtEClassName + "\" for \"" + missingEClasses.getKey().getName() + "\" could not be found.";
				getLogger().info(logMsg);
			}
		}
	}
	
	public String getPackageName(){
		if (metaModelType.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			return ModelConstants.DATA_TYPES;
		}
		else {
			return ModelConstants.PRIMITIVE_TYPES;
		}
	}	
	
	private void replaceAutosarReferences(IResource autosarEcoreFile, ResourceSet resourceSet){
		URI ecoreURI = URI.createPlatformResourceURI(autosarEcoreFile.getFullPath().toString(), true);
		Resource autosarEcoreRes = resourceSet.getResource(ecoreURI, true);
		
		ecoreURI = URI.createFileURI(modelFile.getAbsolutePath().toString());
		Resource targetEcore = Resource.Factory.Registry.INSTANCE.getFactory(ecoreURI).createResource(ecoreURI);
		targetEcore.getContents().add(model);
		
		resourceSet.getResources().add(targetEcore);
		
		EPackage pkg = findEPackageByName("autosarreferences");
		EList<EClassifier> autosarReferenceList = pkg.getEClassifiers();

		if (pkg != null) {			
			TreeIterator<EObject> iter = model.eAllContents();
			while (iter.hasNext()) {
				EObject element = (EObject) iter.next();
				if (element instanceof EReference) {
					EReference eRef = (EReference) element;
					EClassifier eType = eRef.getEType();
					if (autosarReferenceList.contains(eType)) {
						EClassifier autosarElement = getMappedAutosarElement(eType, autosarEcoreRes);
						eRef.setEType(autosarElement);
						eRef.setEOpposite(null);
						logger.info("Replacing AUTOSAR reference " + eType.getName());
					}
				}
			}
					
		}
	}
	
	private EClassifier getMappedAutosarElement(EClassifier eReferenceType, Resource autosarEcoreRes){
		String referenceTypeName = eReferenceType.getName();
		if (referenceTypeName != null) {
			String packageName = autosarMapping.get(referenceTypeName);
			if (packageName != null) {
				 EPackage autosarRootPackage = getRootPackage(autosarEcoreRes);
				 EPackage ePackage = getEPackageByName(autosarRootPackage, packageName);
				 EClassifier eClassifier;
				 if(referenceTypeName.equals("HwElementPrototype") || referenceTypeName.equals("HwElementType") ){
					 eClassifier = getEClassifier(ePackage, "HwElement");
				 }
				 else{				 
					 eClassifier = getEClassifier(ePackage, referenceTypeName);
				 }
				 
				 return eClassifier;
			} else {
				logger.error("No mapping for AUTOSAR reference " + referenceTypeName + " available! The instanceClassName for " + referenceTypeName + " could not set!");
			}
		}
		
		return null;
	}
	
	protected EPackage getRootPackage(Resource ecoreResource){
		List<EPackage> rootEPackageList = EcoreUtil.getEAllRootEPackages(ecoreResource);
		return rootEPackageList.get(0);
	}
	
	
	private EClassifier createPrimitiveTypeFromEClass(EClass eClass) {
		EDataType primitiveType = EcoreFactory.eINSTANCE.createEDataType();
		primitiveType.setName(eClass.getName());
		primitiveType.setInstanceTypeName(eClass.getName());
		primitiveType.getEAnnotations().addAll(eClass.getEAnnotations());
		return primitiveType;
	}

	private EList<EClass> getAllEClassesFromClassifiers(Collection<EClassifier> eClassifiers) {
		EList<EClass> classes = new BasicEList<EClass>();
		for (EClassifier classifier : eClassifiers) {
			if (classifier instanceof EClass) {
				classes.add((EClass) classifier);
			}
		}
		return classes;
	}
}
