package eaadapter.wizard;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import eaadapter.datatypes.MetaModelType;
import eaadapter.presentation.EaEditorPlugin;
import org.eclipse.swt.widgets.Combo;

public class SelectEAPFileWizardPage extends WizardPage {
	private Text eapFilePath;
	private File fileFromDialog;
	private Combo comboTypeOfMetaModel;

	/**
	 * Create the wizard.
	 */
	public SelectEAPFileWizardPage() {
		super("wizardPage");
		setTitle("Select Enterprise Architect file");
		setDescription("Select the Enterprise Architect project file *.eap, which shall be transformed to Ecore");
		setPageComplete(false);
	}

	
	/**
	 * Create contents of the wizard.
	 * @param parent
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new FormLayout());
		
		Label lblEnterpriseArchitectProject = new Label(container, SWT.NONE);
		FormData fd_lblEnterpriseArchitectProject = new FormData();
		fd_lblEnterpriseArchitectProject.top = new FormAttachment(0, 10);
		fd_lblEnterpriseArchitectProject.left = new FormAttachment(0, 10);
		fd_lblEnterpriseArchitectProject.right = new FormAttachment(0, 196);
		lblEnterpriseArchitectProject.setLayoutData(fd_lblEnterpriseArchitectProject);
		lblEnterpriseArchitectProject.setText("Enterprise Architect project file");
		
		eapFilePath = new Text(container, SWT.BORDER);
		FormData fd_filename = new FormData();
		fd_filename.top = new FormAttachment(lblEnterpriseArchitectProject, 6);
		fd_filename.left = new FormAttachment(0, 10);
		eapFilePath.setLayoutData(fd_filename);
		eapFilePath.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				super.keyReleased(e);
				validatePage();
			}
		});
		
		Button btnBrowse = new Button(container, SWT.NONE);
		fd_filename.right = new FormAttachment(btnBrowse, -6);
		btnBrowse.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				FileDialog fileDialog = new FileDialog(getShell());
				fileDialog.setFilterExtensions(new String[]{"*.eap"});
				String result = fileDialog.open();
				if (result != null) {
					fileFromDialog = new File(result);
					eapFilePath.setText(fileFromDialog.getPath());
					validatePage();
				}
			}
		});
		FormData fd_btnBrowse = new FormData();
		fd_btnBrowse.top = new FormAttachment(0, 27);
		fd_btnBrowse.right = new FormAttachment(100, -10);
		btnBrowse.setLayoutData(fd_btnBrowse);
		btnBrowse.setText("Browse...");
		
		Label lblTypeOfMetamodel = new Label(container, SWT.NONE);
		FormData fd_lblTypeOfMetamodel = new FormData();
		fd_lblTypeOfMetamodel.top = new FormAttachment(eapFilePath, 16);
		fd_lblTypeOfMetamodel.left = new FormAttachment(lblEnterpriseArchitectProject, 0, SWT.LEFT);
		lblTypeOfMetamodel.setLayoutData(fd_lblTypeOfMetamodel);
		lblTypeOfMetamodel.setText("Type of Meta-Model");
		
		comboTypeOfMetaModel = new Combo(container, SWT.READ_ONLY);
		FormData fd_comboTypeOfMetaModel = new FormData();
		fd_comboTypeOfMetaModel.top = new FormAttachment(lblTypeOfMetamodel, -3, SWT.TOP);
		fd_comboTypeOfMetaModel.left = new FormAttachment(lblTypeOfMetamodel, 12);
		comboTypeOfMetaModel.setLayoutData(fd_comboTypeOfMetaModel);
		comboTypeOfMetaModel.setItems(availableMetaModelTypes());
		comboTypeOfMetaModel.select(0);		
	}

	private String[] availableMetaModelTypes() {
		return new String[] {MetaModelType.EASTADL.getLiteral(), MetaModelType.SAFE.getLiteral(), MetaModelType.STANDARDUML.getLiteral()};
	}

	public File getSelectedEAPfile() {
		return new File(eapFilePath.getText());
	}
	
	public MetaModelType getSelectedMetaModelType() {
		return MetaModelType.get(comboTypeOfMetaModel.getText());
	}

	private void validatePage() {
		File selectedFile = new File(eapFilePath.getText());
		if (!selectedFile.exists()) {
			setErrorMessage("Selected file does not exist!");
			setPageComplete(false);
		} else if (!isEAPFile(selectedFile)) {
			setErrorMessage("Selected file is not an Enterprise Architect project file");
			setPageComplete(false);
		} else {
			setErrorMessage(null);
			setPageComplete(true);
		}		
	}
	
	/**
	 * checks if given file ends with .eap
	 */
	private boolean isEAPFile(File file) {
		String extension = "";
		String fileName = file.getPath();
		int i = fileName.lastIndexOf('.');
		if (i > 0) {
		    extension = fileName.substring(i+1);
		}
		if (extension.equalsIgnoreCase("eap")) {
			return true;
		} else {
			return false;
		}
	}
}
