/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.templates.internal.util;

import java.util.Iterator;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.codegen.ecore.genmodel.GenClass;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.util.EcoreUtil;

import eaadapter.common.util.ModelConstants;
import eaadapter.common.util.ModelUtil;
import eaadapter.templates.source.EastADLSourceRevisions;
import eaadapter.templates.source.SAFESourceRevisions;

public class GenModels {
	
	public static boolean hasRootUtilitiesGenPackage(GenModel genModel) {
		
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		
		if(value.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			return false;
		}
		
		return true;
	}
	
    public static boolean IsLibJarCreated(GenModel genModel) {
		
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		
		if(value.equals(ModelConstants.METAMODEL_TYPE_EASTADL)){
			return true;
		}
		
		return false;
	}
    
    public static boolean IsLibSrcZipCreated(GenModel genModel) {
		
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		
		if(value.equals(ModelConstants.METAMODEL_TYPE_EASTADL)){
			return true;
		}
		
		return false;
	}
	
	public static String getFileExtension(GenModel genModel){
		
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		
		if(value.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			return "safe";
		}
		else if(value.equals(ModelConstants.METAMODEL_TYPE_EASTADL)){
			return "eaxml";
		}
		
		return "";
	}
	
	/**
	 * Returns the type of Meta-Model, defined in ModelConstants
	 */
	public static String getTypeOfMetaModel(GenModel genModel){
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		return value;
	}	
	
	/**
	 * Returns true if we have an EAST-ADL meta-model
	 */
	public static boolean isEASTADLMetaModel(GenModel genModel) {
		if (getTypeOfMetaModel(genModel).equals(ModelConstants.METAMODEL_TYPE_EASTADL))
			return true;
		return false;
	}
	
	/**
	 * Returns true if we have an SAFE meta-model
	 */
	public static boolean isSAFEMetaModel(GenModel genModel) {
		if (getTypeOfMetaModel(genModel).equals(ModelConstants.METAMODEL_TYPE_SAFE))
			return true;
		return false;
	}	
	
    public static String getNamespacePattern(GenModel genModel){
		
		GenPackage rootGenPackage = getRootGenPackage(genModel);
		EModelElement ecoreModelElement = rootGenPackage.getEcoreModelElement();
		String value = ModelUtil.getEAAdapterAnnotation(ecoreModelElement, ModelConstants.EAADAPTER_METAMODEL_TYPE);
		
		if(value.equals(ModelConstants.METAMODEL_TYPE_SAFE)){
			return "http://safe-project.eu/"+ SAFESourceRevisions.get().major + "\\.[0-9]\\.[0-9]+$";
		}
		
		return "http://east-adl.info/"+ EastADLSourceRevisions.get().major + "\\.[0-9]\\.[0-9]+$";
	}

	public static GenPackage getRootGenPackage(GenModel genModel) {
		return genModel.getGenPackages().get(0);
	}

	public static String getUtilityClassSimpleName(GenModel genModel, String classNamePostfix) {
		return getRootGenPackage(genModel).getPrefix() + classNamePostfix;
	}

	public static String getUtilityClassName(GenModel genModel, String classNamePostfix) {
		return getRootGenPackage(genModel).getUtilitiesPackageName() + "." + getUtilityClassSimpleName(genModel, classNamePostfix); //$NON-NLS-1$
	}

	public static String getQualifiedFactoryName(GenModel genModel) {
		return genModel.getModelName() + ".util." + getFactoryName(genModel); //$NON-NLS-1$
	}

	public static String getFactoryName(GenModel genModel) {
		return getCapModelName(genModel) + "Factory"; //$NON-NLS-1$
	}

	private static String getCapModelName(GenModel genModel) {
		String modelName = genModel.getModelName();
		return modelName.substring(0, 1).toUpperCase() + modelName.substring(1);
	}

	public static String getImportedFactoryName(GenModel genModel) {
		return genModel.getImportedName(getQualifiedFactoryName(genModel));
	}

	public static String getNormalizedModelName(GenModel genModel) {
		String modelName = genModel.getModelName();
		return modelName.substring(0, 1) + modelName.substring(1);
	}

	public static String getModelPluginVersion(GenModel genModel) {
		EPackage rootEPackage = getRootGenPackage(genModel).getEcorePackage();
		String version = EcoreUtil.getAnnotation(rootEPackage, GenModelPackage.eNS_URI, "modelPluginVersion"); //$NON-NLS-1$
		return version != null ? version : "1.0.0"; //$NON-NLS-1$
	}

	public static String getEditPluginVersion(GenModel genModel) {
		EPackage rootEPackage = getRootGenPackage(genModel).getEcorePackage();
		String version = EcoreUtil.getAnnotation(rootEPackage, GenModelPackage.eNS_URI, "editPluginVersion"); //$NON-NLS-1$
		return version != null ? version : "1.0.0"; //$NON-NLS-1$
	}

	public static String findInterfacePackageName(GenModel genModel, String className) {
		String interfacePackageName = ""; //$NON-NLS-1$
		for (Iterator<EObject> iterator = genModel.eAllContents(); iterator.hasNext();) {
			EObject type = iterator.next();
			if (type instanceof GenClass) {
				GenClass genClass = (GenClass) type;
				if (genClass.getName().equals(className)) {
					interfacePackageName = genClass.getGenPackage().getInterfacePackageName();
					break;
				}
			}

		}
		return interfacePackageName;
	}

	public static GenClass findGenClass(GenModel genModel, String className) {
		GenClass genClass = null;
		for (Iterator<EObject> iterator = genModel.eAllContents(); iterator.hasNext();) {
			EObject type = iterator.next();
			if (type instanceof GenClass) {
				GenClass genClazz = (GenClass) type;
				if (genClazz.getName().equals(className)) {
					genClass = genClazz;
					break;
				}
			}
		}
		return genClass;
	}
	
	public static String getEastADLReleaseNamespace(GenModel genModel) {
		String nsURI = getRootGenPackage(genModel).getNSURI();
		return nsURI.substring(0, nsURI.lastIndexOf("/")); //$NON-NLS-1$ 
	}
	
	public static String getEastADLMMDescriptorID(GenModel genModel) {
		return "org.eatop.eel." + getRootGenPackage(genModel).getPackageName(); //$NON-NLS-1$
	}
	
	
	/**
	 * creates the "id" attribute for the extension point
	 * "org.eclipse.sphinx.emf.validation.registration"
	 */
	public static String createValidationId(GenModel genModel) {
		return String.format("%s.%s.validation", genModel.getModelPluginID(), getRootGenPackage(genModel).getPrefix());
		
	}
	
	/**
	 * creates the "NSURI" attribute for the extension point
	 * "org.eclipse.sphinx.emf.validation.registration"
	 */	
	public static String createValidationNSURI(GenModel genModel) {
		GenClass referrable = findGenClass(genModel, "Referrable");
		Assert.isNotNull(referrable);
		return referrable.getGenPackage().getNSURI();
	}
	
	/**
	 * creates the "filter" attribute for the extension point
	 * "org.eclipse.sphinx.emf.validation.registration"
	 */
	public static String createValidationFilter(GenModel genModel) {
		if (isEASTADLMetaModel(genModel)) {
			return "_" + EastADLSourceRevisions.get().major + EastADLSourceRevisions.get().minor;
		}
		else if (isSAFEMetaModel(genModel)) {
			return "_" + SAFESourceRevisions.get().major + SAFESourceRevisions.get().minor;			
		}
		return "_undefined";
	}	
	
	/**
	 * creates the "Name" attribute for the extension point
	 * "org.eclipse.sphinx.emf.validation.registration"
	 */
	public static String createValidationName(GenModel genModel) {
		if (isEASTADLMetaModel(genModel)) {
			return "EAST-ADL " + EastADLSourceRevisions.get().major + EastADLSourceRevisions.get().minor;
		}
		else if (isSAFEMetaModel(genModel)) {
			return "SAFE " + SAFESourceRevisions.get().major + SAFESourceRevisions.get().minor;			
		}
		return "undefined";
	}
	
	/**
	 * creates the "class" attribute for the extension point
	 * "org.eclipse.sphinx.emf.validation.registration"
	 */
	public static String createValidationClass(GenModel genModel) {
		GenClass referrable = findGenClass(genModel, "Referrable");
		Assert.isNotNull(referrable);
		return referrable.getQualifiedInterfaceName();
	}	
}
