/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */

package eaadapter.util;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import eaadapter.EAAttribute;
import eaadapter.EAConnector;
import eaadapter.EAElement;
import eaadapter.EARepository;

public class Util {	
	
	/**
	 * Returns an EAElement based on its name
	 * @param name
	 * @param model
	 * @return
	 */
	public static EAElement findEAElementByName(String name, EARepository model) {
		TreeIterator<EObject> iter = model.eAllContents();
		while (iter.hasNext()) {
			EObject element = (EObject) iter.next();
			if (element instanceof EAElement) {
				if (((EAElement) element).getName().equalsIgnoreCase(name))
					return (EAElement) element;
			}
		}
		return null;		
	}

	/**
	 * Get an attribute of an EAElement based on its name
	 * @param name
	 * @param element
	 * @param model
	 * @return
	 */
	public static EAAttribute getEAAttributeForEAElement(String name, EAElement element, EARepository model) {
		for (EAAttribute attribute : element.getAttributes()) {
			if (attribute.getName().equals(name))
				return attribute;
		}
		return null;
	}

	/**
	 * returns the connector of an EAElement where either the clientEnd's roleName = name
	 * or supplierEnd's rolename = name
	 * @param rolename
	 * @param element
	 * @param eaModel
	 * @return
	 */
	public static EAConnector getEAConnectorByRolenameForEAElement(String rolename, EAElement element, EARepository eaModel) {
		for (EAConnector connector : element.getConnectors()) {
			if (connector.getSupplierEnd().getRole().equals(rolename)) 
				return connector;
			
			if (connector.getClientEnd().getRole().equals(rolename))
				return connector;
		}
		return null;
	}
	
	/**
	 * Finds an EPackage for the given name within the root
	 * @param name
	 * @param root
	 * @return
	 */
	public static EPackage findEPackageByName(String name, EPackage root) {
		TreeIterator<EObject> iter = root.eAllContents();
		while (iter.hasNext()) {
			EObject element = (EObject) iter.next();
			if (element instanceof EPackage) {
				if (((EPackage) element).getName().equals(name))
					return (EPackage) element;
			}
		}
		return null;		
	}
		
}
