/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd.common;

import eaadapter.common.logging.Logger;
import eaadapter.common.logging.LoggerImpl;

public class ModelInformation {

    private static final String MSG_FAILED_TO_LOAD_RELEASE_INFORMATION = 
            "Could not load release information! Exiting workflow!";
    
    private String release = null;
    private String nsuri = null;
    private String nsprefix = null;
    
    private Logger logger = new LoggerImpl();

    ModelInformation() {
        super();
    }

    protected void checkValues() {
        // Check whether release can be determined
        if (release == null || release.isEmpty()) {

            // no release information specified, so we don't know which template
            // set to use!
            logger.error(MSG_FAILED_TO_LOAD_RELEASE_INFORMATION);
            System.exit(0);
        }
    }


    /**
     * Returns the metamodel version instance which is similar to the number
     * from {@link #getMMVersion()}
     * 
     * @return The metamodel version instance which provides some additional
     *         helper functionality. Not <code>null</code>.
     */
    public MetamodelVersion getMetamodelVersion() {
        MetamodelVersion result = null;
        if (release != null) {
            result = MetamodelVersion.getNewInstance(release);
        }
        if (result == null) {
            result = MetamodelVersion.UNSPECIFIED;
        }
        return result;
    }

    /**
     * Returns the namespace uri.
     * 
     * @return The current namespace uri.
     */
    public String getNSUri() {
        return nsuri;
    }

    /**
     * Returns the namespace prefix.
     * 
     * @return The current namespace prefix.
     */
    public String getNSPrefix() {
        return nsprefix;
    }

    protected String getNsuri() {
        return nsuri;
    }

    protected void setNsuri(String nsuri) {
        this.nsuri = nsuri;
    }

    protected String getNsprefix() {
        return nsprefix;
    }

    protected void setNsprefix(String nsprefix) {
        this.nsprefix = nsprefix;
    }
    
    protected void setRelease(String release) {
        this.release = release;
    }

}
