package eaadapter.xsd.output;

import java.util.Locale;

import org.stringtemplate.v4.AttributeRenderer;

import eaadapter.xsd.model.EAXSDCardinality;

public class EAXSDCardinalityRenderer implements AttributeRenderer {

    private static final String UNBOUNDED = "unbounded";
    private static final String REDUCED = "reduced";
    private static final String UPPER = "upper";
    private static final String LOWER = "lower";

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString(Object value, String formatString, Locale locale) {

        EAXSDCardinality cardinality = (EAXSDCardinality) value;
        Integer upperMult = cardinality.getUpperMultiplicity();
        Integer lowerMult = cardinality.getLowerMultiplicity();

        if (upperMult == null && lowerMult == null) {
            return "";
        }

        if (formatString.equals(UPPER)) {
            if (upperMult.intValue() == -1) {
                return UNBOUNDED;
            }
            return String.valueOf(upperMult);
        }

        if (formatString.equals(LOWER)) {
            return String.valueOf(lowerMult);
        }

        if (formatString.equals(REDUCED)) {

            int lower = lowerMult.intValue();
            if (lower > 1) {
                lower = 1;
            }
            return String.valueOf(lower);

        }

        if (lowerMult.equals(upperMult)) {
            return lowerMult.toString();
        }

        if (upperMult.intValue() == -1) {
            return String.format("%d..*", lowerMult);
        } else {
            return String.format("%d..%d", lowerMult, upperMult);
        }
    }
}
