package eaadapter.xsd.output;

import java.util.Locale;

import org.stringtemplate.v4.AttributeRenderer;

import eaadapter.common.util.ModelConstants;
import eaadapter.xsd.model.EAXSDType;

public class EAXSDTypeRenderer implements AttributeRenderer {

    private static final String PARAM_NO_PREFIX = "noprefix";

    private String nsPrefix;
    private boolean iRefSuffix;
    private boolean simpleSuffix;

    public EAXSDTypeRenderer(String prefix, boolean enableSimpleSuffix,
            boolean enableIRefSuffix) {

        nsPrefix = ModelConstants.DEFAULT_NSPREFIX;
        if (prefix != null) {
            String tmpPrefix = prefix.trim();
            if (tmpPrefix.length() == 0) {
                nsPrefix = tmpPrefix;
            }
        }

        simpleSuffix = enableSimpleSuffix;
        iRefSuffix = enableIRefSuffix;

    }

    /**
     * {@inheritDoc }
     */
    @Override
    public String toString(Object value, String formatString, Locale locale) {
        EAXSDType type = (EAXSDType) value;
        String prefix = "";
        String suffix = "";

        if (formatString == null || !formatString.equals(PARAM_NO_PREFIX)) {
            if (type.isSimple() && !type.isCustom()) {
                prefix = "xsd:";
            } else {
                prefix = String.format("%s:", nsPrefix);
            }
        }
        
        if (iRefSuffix && type.isInstanceref()) {
            if (!type.getXmlname().endsWith("-IREF")) {
                suffix = "-IREF";
            }
        } else if (simpleSuffix && type.isSimplesuffix()) {
            suffix = "--SIMPLE";
        }

        return String.format("%s%s%s", prefix, type.getXmlname(), suffix);
    }
}
