/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd.stringtemplates.adapters;

import javax.xml.bind.annotation.adapters.XmlAdapter;

/**
 * Simple adapter which provides the capability to convert a textual value
 * describing a multiplicity into a numerical one. Note that the negative value
 * <code>-1</code> indicates an unbounded value.
 */
public class MultiplicityAdapter extends XmlAdapter<String, Integer> {

    /**
     * {@inheritDoc}
     */
    @Override
    public String marshal(Integer v) throws Exception {
        if (v.intValue() == -1) {
            return "*";
        } else {
            return v.toString();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Integer unmarshal(String v) throws Exception {
        if ("*".equals(v)) {
            return Integer.valueOf(-1);
        } else {
            return Integer.valueOf(v);
        }
    }
}