/**
 * <copyright>
 * 
 * Copyright (c) Continental AG and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 
 * which accompanies this distribution, and is
 * available at http://www.eclipse.org/org/documents/epl-v10.php
 * 
 * Contributors: 
 *     Continental AG - Initial API and implementation
 * 
 * </copyright>
 */
package eaadapter.xsd.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

public class IOUtil {
    
    /**
     * Recursively deletes the supplied file.
     *
     * @param file
     *           The file entry which may be a file or a directory. Not <code>null</code>.
     *           
     * @return   <code>true</code> <=> The deletion succeeded.
     */
    public static final boolean delete(File file)    {
        if (file == null)  {
            throw new IllegalArgumentException(
                    "The parameter '%s' is not allowed to be <null>.");
        }
        
        if (file.isFile()) {
            if (!file.delete()) {
                return false;
            }
        } else if (file.isDirectory()) {
            File[] children = file.listFiles();
            for (File child : children) {
                if (!delete (child)) {
                    return false;
                }
            }
            if (!file.delete()) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Writes the given content into the given file.
     * 
     * @param file 
     * @param content
     *            
     * @throws IOException
     */
    public static final void write(File file, String content)
            throws IOException {
        
        OutputStreamWriter w = null;
        try {
            w = new OutputStreamWriter(new FileOutputStream(file), "UTF8");
            w.write(content);
            w.flush();
        } finally {
            try {
                w.close();
            } catch (IOException e) {
                // ignore
            }
        }
    }

}
